# _*_ coding: utf-8 _*_
#
# hspylib v0.11.39
#
# Package: main.hspylib.modules.cli.icons.font_awesome
"""Package initialization."""

__all__ = [
  'awesome', 
  'dashboard_icons', 
  'form_icons', 
  'widget_icons'
]
__version__ = '0.11.39'
