# _*_ coding: utf-8 _*_
#
# hspylib v0.11.39
#
# Package: main.hspylib.modules.qt.promotions
"""Package initialization."""

__all__ = [
  'hcombobox', 
  'hconsole', 
  'hframe', 
  'hlabel', 
  'hlistwidget', 
  'htablemodel', 
  'htableview'
]
__version__ = '0.11.39'
