# _*_ coding: utf-8 _*_
#
# hspylib v0.11.39
#
# Package: main.hspylib.modules.cli
"""Package initialization."""

__all__ = [
  'application', 
  'icons', 
  'keyboard', 
  'tui', 
  'vt100'
]
__version__ = '0.11.39'
