# _*_ coding: utf-8 _*_
#
# hspylib v0.11.39
#
# Package: main.hspylib.addons.appman
"""Package initialization."""

__all__ = [
  'app_extension', 
  'app_type', 
  'appman', 
  'templates'
]
__version__ = '0.11.39'
